/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.io;

import java.io.PrintWriter;

//import org.opengion.fukurou.util.StringUtil;									// 8.5.2.0 (2023/07/14) Delete
import org.opengion.hayabusa.db.DBTableModel;

/**
 * 加工なし区切り文字指定ﾃﾞｰﾀの書き出しｸﾗｽです。
 *
 * DefaultTableWriter を継承して、ﾃﾞｰﾀの出力部のみｵｰﾊﾞｰﾗｲﾄﾞして、
 * ﾃﾞｰﾀそのものを加工なしで、出力します。
 * 本来は、DefaultTableWriter の出力形態のはずですが、過去の互換性との関係で、
 * なまﾃﾞｰﾀを出力するｸﾗｽを、追加作成しました。
 *
 * @og.rev 3.1.9.1 (2003/05/20) 新規作成
 * @og.group ﾌｧｲﾙ出力
 *
 * @version	4.0
 * @author	Kazuhiko Hasegawa
 * @since	JDK5.0,
 */
public class TableWriter_Data extends TableWriter_Default {
	/** このﾌﾟﾛｸﾞﾗﾑのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "8.5.2.0 (2023/07/14)" ;

	/**
	 * ﾃﾞﾌｫﾙﾄｺﾝｽﾄﾗｸﾀｰ
	 *
	 * @og.rev 6.4.2.0 (2016/01/29) PMD refactoring. Each class should declare at least one constructor.
	 */
	public TableWriter_Data() { super(); }		// これも、自動的に呼ばれるが、空のﾒｿｯﾄﾞを作成すると警告されるので、明示的にしておきます。

	/**
	 * PrintWriter に DBTableModelのﾃｰﾌﾞﾙ情報を書き込みます。
	 *
	 * @og.rev 3.7.0.2 (2005/02/14) 行番号情報を、出力する(true)/しない(false)を指定
	 * @og.rev 3.8.0.1 (2005/06/17) DBTypeが NVAR の場合は、元のUnicodeに戻します。
	 * @og.rev 5.1.6.0 (2010/05/01) DbType の初期値(dbType)を利用する。
	 * @og.rev 5.2.1.0 (2010/10/01) useRenderer 対応
	 * @og.rev 5.2.2.0 (2010/11/01) 改行、ﾀﾞﾌﾞﾙｸｵｰﾄ等の処理
	 * @og.rev 6.0.1.2 (2014/08/08) ｶﾗﾑ飛ばしできる機能を追加
	 * @og.rev 6.0.4.0 (2014/11/28) ﾃﾞｰﾀ出力用のﾚﾝﾃﾞﾗｰ
	 * @og.rev 8.5.2.0 (2023/07/14) 一部の機能廃止による修正(問合・ﾄﾗﾌﾞﾙ 0200010980)
	 *
	 * @param	table	DBTableModelｵﾌﾞｼﾞｪｸﾄ
	 * @param	writer	PrintWriterｵﾌﾞｼﾞｪｸﾄ
	 */
	@Override
	protected void writeData( final DBTableModel table,final PrintWriter writer ) {
		final int numberOfRows = table.getRowCount();
		final String separator = getSeparator();
		final boolean useNumber = isUseNumber();
		final boolean useRenderer = isUseRenderer();		// 5.2.1.0 (2010/10/01)

		for( int row=0; row<numberOfRows; row++ ) {
			if( useNumber ) {
				writer.print( String.valueOf( row+1 ) );
				writer.print( separator );
			}

			for( int i=0; i<numberOfColumns; i++ ) {
				if( i != 0 ) { writer.print( separator ); }
				final int clm = clmNo[i];
				if( clm < 0 ) { continue; }					// 6.0.1.2 (2014/08/08) ｶﾗﾑ飛ばし

				String val = table.getValue(row,clm);
//				if( dbType[i] == NVAR ) {										// 8.5.2.0 (2023/07/14) Delete
//					val = StringUtil.getReplaceEscape( val );					// 8.5.2.0 (2023/07/14) Delete
//				}																// 8.5.2.0 (2023/07/14) Delete
				// 5.2.1.0 (2010/10/01) useRenderer 対応
//				else if( useRenderer ) {										// 8.5.2.0 (2023/07/14) Modify
				if( useRenderer ) {
					// 6.0.4.0 (2014/11/28) ﾃﾞｰﾀ出力用のﾚﾝﾃﾞﾗｰ
					val = dbColumn[clm].getWriteValue( val );
				}

				writer.print( quotation2( val ) );			// 5.2.2.0 (2010/11/01) 改行、ﾀﾞﾌﾞﾙｸｵｰﾄ等の処理
			}
			writer.println();
		}
	}
}
